@@ -10,9 +10,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <vga.h>
-#include <vgagl.h>
-#include <vgamouse.h>
+#include "zgv_io.h"
 #include "3deffects.h"
 #include "font.h"
 #include "readnbkey.h"
