@@ -1,5 +1,5 @@
-/* Zgv v3.1 - GIF, JPEG and PBM/PGM/PPM viewer, for VGA PCs running Linux.
- * Copyright (C) 1993-1998 Russell Marks. See README for license details.
+/* zgv 5.7 - GIF, JPEG and PBM/PGM/PPM viewer, for VGA PCs running Linux.
+ * Copyright (C) 1993-2003 Russell Marks. See README for license details.
  *
  * mousecur.c - mouse cursor routines.
  *
@@ -10,9 +10,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <vga.h>
-#include <vgagl.h>
-#include <vgamouse.h>
+#include "zgv_io.h"
 #include "rc_config.h"
 #include "rcfile.h"
 #include "zgv.h"
@@ -37,9 +35,9 @@ static int screen_width,screen_height,screen_bytepp;
 #define MOUSECUR_YSIZE	16
 #define MOUSECUR_PIXMAP_SIZE	(MOUSECUR_XSIZE*MOUSECUR_YSIZE)
 
-/* next two pixmaps are *3 as must be able to handle up to 24-bit modes */
-static unsigned char mouseptr_save[MOUSECUR_PIXMAP_SIZE*3];
-static unsigned char mouseptr_pixmap[MOUSECUR_PIXMAP_SIZE*3];
+/* next two pixmaps are *4 as must be able to handle up to 32-bpp modes */
+static unsigned char mouseptr_save[MOUSECUR_PIXMAP_SIZE*4];
+static unsigned char mouseptr_pixmap[MOUSECUR_PIXMAP_SIZE*4];
 static unsigned char mouseptr_pixmap_orig[MOUSECUR_PIXMAP_SIZE]=
   {
   /* 1 is black, 2 is white, 0 is transparent */
@@ -89,20 +87,11 @@ static unsigned char mouseptr_pixmap_narrow_orig[MOUSECUR_PIXMAP_SIZE/2]=
 void mousecur_init(int blackcol,int whitecol)
 {
 int f,val;
-int cols=vga_getcolors();
 int mode=vga_getcurrentmode();
 
 if(!has_mouse) return;
 
-switch(cols)
-  {
-  case 16777216:
-    screen_bytepp=3; break;
-  case 65536: case 32768:
-    screen_bytepp=2; break;
-  default:
-    screen_bytepp=1;
-  }
+screen_bytepp=vga_getmodeinfo(mode)->bytesperpixel;
 
 screen_width =vga_getxdim();
 screen_height=vga_getydim();
@@ -139,6 +128,12 @@ for(f=0;f<(usenarrow?MOUSECUR_PIXMAP_SIZE/2:MOUSECUR_PIXMAP_SIZE);f++)
   
   switch(screen_bytepp)
     {
+    case 4:
+      mouseptr_pixmap[f*4  ]=(val&255);
+      mouseptr_pixmap[f*4+1]=(val>>8);
+      mouseptr_pixmap[f*4+2]=(val>>16);
+      mouseptr_pixmap[f*4+3]=0;
+      break;
     case 3:
       mouseptr_pixmap[f*3  ]=(val&255);
       mouseptr_pixmap[f*3+1]=(val>>8);
@@ -175,7 +170,7 @@ if(savey>screen_height-saveh)
 /* we need to save existing data, then draw the mouse. */
 if(usegl)
   {
-  /* gl-using 8/15/16/24-bit version */
+  /* gl-using 8/15/16/24/32-bpp version */
   gl_getbox(savex,savey,savew,saveh,mouseptr_save);
   /* since there's no gl_putboxpartmask, and clipping doesn't work for
    * gl_putboxmask, we have to do either masking or clipping ourselves.
