@@ -1,5 +1,5 @@
-/* Zgv v2.7 - GIF, JPEG and PBM/PGM/PPM viewer, for VGA PCs running Linux.
- * Copyright (C) 1993-1995 Russell Marks. See README for license details.
+/* zgv 5.2 - GIF, JPEG and PBM/PGM/PPM viewer, for VGA PCs running Linux.
+ * Copyright (C) 1993-2000 Russell Marks. See README for license details.
  *
  * rcfile.h - protos for rcfile.c
  */
@@ -10,5 +10,4 @@ extern struct zgv_config cfg;
 extern void getconfig(void);
 extern void fixconfig(void);
 extern int parsecommandline(int argc,char **argv);
-extern void usage_help(void);
 extern int modematch(int x, int y, int bpp);
