@@ -6,26 +6,13 @@
  * BMP support by Carsten Engelmann (cengelm@gwdg.de)
  */
 
-#define TRUE 1
-#define FALSE 0
-
-/* BITMAPFILEHEADER
- *
- * Bitmap File Information
- *
- * The BITMAPFILEHEADER data structure contains information about the type,
- * size, and layout of a device-independent bitmap (DIB) file.
+/* these are no longer read directly from the file, so
+ * endianness, short/int lengths, and struct packing shouldn't
+ * be a problem any more.
+ * -rjm 2001 May 25.
  */
-typedef struct BITMAPFILEHEADER
-{
-  short   bfType;
-  int     bfSize;
-  short   bfReserved1;
-  short   bfReserved2;
-  int     bfOffBits;
-} BITMAPFILEHEADER;
 
-typedef struct BITMAPINFOHEADER
+typedef struct
 {
   unsigned int  biSize;
   unsigned int  biWidth;
@@ -40,7 +27,7 @@ typedef struct BITMAPINFOHEADER
   unsigned int  biClrImportant;
 } BITMAPINFOHEADER;
 
-typedef struct BITMAPCOREHEADER
+typedef struct
 {
   unsigned int  bcSize;
   unsigned short  bcWidth;
