@@ -13,15 +13,54 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
-#include <vga.h>
-#include <vgamouse.h>
-#include <vgakeyboard.h>	/* for SCANCODE_{F1,F11} */
-#include <linux/kd.h>
+#include "zgv_io.h"
 #include "zgv.h"
 #include "readnbkey.h"
 #include "mousecur.h"
 
 
+#ifndef BACKEND_SVGALIB
+
+/* generic redirector for non-svgalib backends. Currently there's
+ * no provision for mouse support in these.
+ */
+
+int readnbkey(int ttyfd)
+{
+return(zgv_io_readnbkey());	/* defined in zgv_io.c */
+}
+
+int wait_for_keys_or_mouse(int fd)
+{
+return(zgv_io_waitkey());	/* defined in zgv_io.c */
+}
+
+int mousecur_wait_for_keys_or_mouse(int fd)
+{
+return(zgv_io_waitkey());
+}
+
+void click_update(void) {}
+
+void save_mouse_pos(void) {}
+
+void restore_mouse_pos(void) {}
+
+void restore_mouse_pos_with_size(int width,int height) {}
+
+int is_end_click_left(void) { return 0; }
+int is_end_click_right(void) { return 0; }
+int is_start_click_left(void) { return 0; }
+int is_start_click_right(void) { return 0; }
+
+
+#else	/* BACKEND_SVGALIB */
+
+
+#include <vgakeyboard.h>	/* for SCANCODE_{F1,F11} */
+#include <linux/kd.h>
+
+
 /* saved mx/my in in range 0..MPOS_SAVE_MAX. */
 #define MPOS_SAVE_MAX	16383
 
@@ -355,3 +394,5 @@ if(!(old_click_status&MOUSE_RIGHTBUTTON) &&
   return(1);
 return(0);
 }
+
+#endif	/* BACKEND_SVGALIB */
