@@ -4,7 +4,5 @@
  * readpng.h - external prototypes for readpng.c
  */
 
-#ifdef PNG_SUPPORT
 extern void aborted_file_png_cleanup(void);
 extern int read_png_file(char *filename, hffunc howfarfunc, byte **palette);
-#endif
