@@ -1,5 +1,5 @@
-/* zgv 5.5 - GIF, JPEG and PBM/PGM/PPM viewer, for VGA PCs running Linux.
- * Copyright (C) 1993-2001 Russell Marks. See README for license details.
+/* zgv 5.7 - GIF, JPEG and PBM/PGM/PPM viewer, for VGA PCs running Linux.
+ * Copyright (C) 1993-2003 Russell Marks. See README for license details.
  *
  * vgadisp.c - VGA-specific display routines.
  */
@@ -11,9 +11,8 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <sys/file.h>
-#include <vga.h>
-#include <vgagl.h>
-#include <vgamouse.h>
+#include <sys/time.h>
+#include "zgv_io.h"
 #include "zgv.h"
 #include "magic.h"
 #include "readgif.h"
@@ -160,8 +159,8 @@ unsigned char vkcache[32768],vkcache_valid[32768];
 #define FIX_TO_EIGHT_BIT	1
 #define FIX_TO_HIGHCOLOUR	2
 
-byte *theimage;   /* quite slow if passed as parameter */
-int width,height,numcols; /* same here */
+byte *theimage;
+int width,height,numcols;
 byte *image_palette;
 
 double contrast=1.0;  /* note that double contrast is in fact 2 :-) */
@@ -279,7 +278,7 @@ int closest(int r, int g, int b);
 void samecentre(int *ppx, int *ppy, int newscale, int oldpx, int oldpy, int oldscale);
 void show_dimensions(int px, int py, int scaling);
 void orient_change_state(int from,int to,int clear_if_rot);
-void animate_gif(int orient_state);
+int animate_gif(int orient_state);
 
 
 
@@ -606,8 +605,9 @@ for(md_ptr=modedesc;md_ptr->mode;md_ptr++)
   h=md_ptr->height;
   
   /* has to be >= pic size, but <= current best-fit mode. */
-  if(w>=width && h>=height && w<=newmode_w && h<=newmode_h)
-    newmode=md_ptr->mode,newmode_w=w,newmode_h=h;
+  if(w+cfg.deltamodefit>=width && h+cfg.deltamodefit>=height)
+    if(w<=newmode_w && h<=newmode_h)
+      newmode=md_ptr->mode,newmode_w=w,newmode_h=h;
   
   /* also track biggest mode as backup in case no mode fits it all. */
   if(w>=bigmode_w && h>=bigmode_h)
@@ -1076,6 +1076,7 @@ int slideshow_pause=0;
 static int orient_lastpicexit_state=0;	/* last picture orientation state */
 int orient_current_state;		/* current picture orientation state */
 int vkcache_reinit=0;
+int doing_auto_animate=0;
 
 wait_for_foreground();
 
@@ -1174,13 +1175,21 @@ if(orient_override)
   orient_current_state=orient_override_state;
   }
 
-redrawgif(px,py,npx,npy);
+if(cfg.auto_animate && gif_delaycount>=2 && pixelsize==1)
+  doing_auto_animate=1;
+else
+  redrawgif(px,py,npx,npy);
 
 while(!quitshow && (repeat_sig!=1 || rb_menu_mode || slideshow_pause))
   {
-  if(rb_menu_mode) mousecur_on();
-  key=wait_for_keys_or_mouse(zgv_ttyfd);
-  if(rb_menu_mode) mousecur_off();
+  if(doing_auto_animate)
+    key='e',quitshow=1;		/* do animation only, then exit */
+  else
+    {
+    if(rb_menu_mode) mousecur_on();
+    key=wait_for_keys_or_mouse(zgv_ttyfd);
+    if(rb_menu_mode) mousecur_off();
+    }
   
   opx=px; opy=py; redraw=0;
   
@@ -1202,7 +1211,7 @@ while(!quitshow && (repeat_sig!=1 || rb_menu_mode || slideshow_pause))
       case 'N':		key=128+'n'; break;
       }
     }
-  
+
   if(rb_menu_mode)
     {
     /* we deal with keys ourselves... */
@@ -1299,7 +1308,7 @@ while(!quitshow && (repeat_sig!=1 || rb_menu_mode || slideshow_pause))
   if(key) scrolling_with_mouse=0;
   
   /* mode selection - note that F11=shift-F1 and F12=shift-F2 */
-  if(key=='0' || (key>='5' && key<='8') ||
+  if(key=='0' || key=='^' || (key>='5' && key<='8') ||
      (key>=RK_F1 && key<=RK_F10) || (key>=RK_SHIFT_F1 && key<=RK_SHIFT_F10))
     {
     struct modedesc_tag *md_ptr;
@@ -1368,6 +1377,9 @@ while(!quitshow && (repeat_sig!=1 || rb_menu_mode || slideshow_pause))
           {
           filterpal(palette),setpalvec(0,256,palrgb);
           if(curvgamode==G640x480x16) redraw=1;
+#ifndef BACKEND_SVGALIB
+          redraw=1;
+#endif
           }
         break;
       
@@ -1458,6 +1470,10 @@ while(!quitshow && (repeat_sig!=1 || rb_menu_mode || slideshow_pause))
           setpalvec(0,256,palrgb);	/* else change onscreen palette */
         /* 640x480 16-colour needs *both*! */
         if(curvgamode==G640x480x16) redraw=1;
+#ifndef BACKEND_SVGALIB
+        /* we assume the same for non-svgalib backends, too */
+        redraw=1;
+#endif
         /* any palette change will invalidate the vkludge cache.
          * Set flag to say we should reinit the cache when we next redraw.
          */
@@ -1479,17 +1495,36 @@ while(!quitshow && (repeat_sig!=1 || rb_menu_mode || slideshow_pause))
       case 'O': case 'H': case RK_CURSOR_LEFT:  px-=BIGSTP; break;
       case 'P': case 'L': case RK_CURSOR_RIGHT: px+=BIGSTP; break;
       case 'e':		/* animate */
-        if(cfg.repeat_timer) alarm(0);		/* cancel for this image */
+        {
+        int ret;
         
-        animate_gif(orient_current_state);
+        if(cfg.repeat_timer || tagview_mode)
+          alarm(0);		/* cancel timeout for this image */
+        
+        ret=animate_gif(orient_current_state);
+
+        if(cfg.repeat_timer || tagview_mode)
+          repeat_sig=0;			/* clear it in case */
+        
+        if(doing_auto_animate)	/* no need for redraw */
+          {
+          switch(ret)	/* change file if requested */
+            {
+            case -1:	goto do_prev_file;
+            case 1:	goto do_next_file;
+            case 2:	goto do_next_and_tag;
+            }
+
+          break;	/* else just exit switch(key) */
+          }
         
         /* need clear as (if zoomed) full pic will take less space,
          * or (if rotated) pic effectively has different shape.
          */
         vga_clear();
         redraw=1;
-        if(cfg.repeat_timer) repeat_sig=0;	/* clear it in case */
         break;
+        }
       case 'm':		/* mirror */
         fx_mirror();
         orient_current_state=orient_state_mirror[orient_current_state];
@@ -1570,10 +1605,13 @@ while(!quitshow && (repeat_sig!=1 || rb_menu_mode || slideshow_pause))
       case 'P'-0x40:
         pic_incr=-1; quitshow=1; break;
       case RK_ENTER:
+      do_next_file:
 	pic_incr=1; quitshow=2; break;
       case ' ':
+      do_next_and_tag:
 	pic_incr=1; quitshow=3; break;
       case 127: case 8:
+      do_prev_file:
 	pic_incr=-1; quitshow=2; break;
       case 'c': case 'C':
         if(curvgamode==G640x480x16)
@@ -1595,6 +1633,10 @@ while(!quitshow && (repeat_sig!=1 || rb_menu_mode || slideshow_pause))
         quitshow=1;
       }
     }
+
+  /* if an auto-animate finished, exit */
+  if(doing_auto_animate)
+    break;
       
   if(!allow_zoom())
     {
@@ -1703,13 +1745,16 @@ viewer_menu_data[f++].active=mode_is_usable_now(G800x600x256);
 viewer_menu_data[f++].active=mode_is_usable_now(G1024x768x256);
 viewer_menu_data[f++].active=mode_is_usable_now(G640x480x32K);
 viewer_menu_data[f++].active=mode_is_usable_now(G640x480x64K);
-viewer_menu_data[f++].active=mode_is_usable_now(G640x480x16M);
+viewer_menu_data[f++].active=
+  mode_is_usable_now(G640x480x16M) || mode_is_usable_now(G640x480x16M32);
 viewer_menu_data[f++].active=mode_is_usable_now(G800x600x32K);
 viewer_menu_data[f++].active=mode_is_usable_now(G800x600x64K);
-viewer_menu_data[f++].active=mode_is_usable_now(G800x600x16M);
+viewer_menu_data[f++].active=
+  mode_is_usable_now(G800x600x16M) || mode_is_usable_now(G800x600x16M32);
 viewer_menu_data[f++].active=mode_is_usable_now(G1024x768x32K);
 viewer_menu_data[f++].active=mode_is_usable_now(G1024x768x64K);
-viewer_menu_data[f++].active=mode_is_usable_now(G1024x768x16M);
+viewer_menu_data[f++].active=
+  mode_is_usable_now(G1024x768x16M) || mode_is_usable_now(G1024x768x16M32);
 
 rbm_set_active_flag(viewer_menu_data,"grey/colour",(curvgamode==G640x480x16));
 
@@ -2058,12 +2103,14 @@ else
     switch(scrnpixelsize)
       {
       case 3:
+#ifdef BACKEND_SVGALIB
         if(!doing_hicol_bc)
           {
           gl_putboxpart(x_add,y_add,(scrnwide<width)?scrnwide:width,
 	  	(scrnhigh<height)?scrnhigh:height,width,height,theimage,px,py);
           break;
           }
+#endif
         /* otherwise, FALLS THROUGH */
       
       default:
@@ -2121,6 +2168,10 @@ switch(scrnpixelsize)
     }
     break;
     
+  case 3:	/* 24-bit, on non-svgalib backends */
+    gl_putbox(x,y,len/3,1,ptr);
+    break;
+    
   case 2:	/* 15/16-bit */
     {
     unsigned short *bufref=buf;
@@ -2769,7 +2820,14 @@ if(orient_state_mirror[orient_state_flip[state]]!=to)
 }
 
 
-void animate_gif(int orient_state)
+/* returns:
+ * -1 if we want to go to previous image,
+ *  1 if we want to go to next image,
+ *  2 if we want to tag this image and go to next,
+ * else 0.
+ * (non-zero returns only meaningful when auto-animating)
+ */
+int animate_gif(int orient_state)
 {
 static int orient_to_movetype[8]={0,1,1,0,3,2,3,2};	/* see below */
 int frame,delay,key,f;
@@ -2783,9 +2841,12 @@ struct itimerval itv;
 int num_images=gif_delaycount;
 int movetype;
 int offset,startoffset;
+unsigned int tagtimeout=(tagview_mode?cfg.tag_timer*100:0);
+unsigned int hundredths=0;
+int ret=0;
 
 if(num_images<2 || pixelsize!=1)
-  return;
+  return(0);
 
 /* the direction and axis of movement along the image between
  * frames depends on the orientation. The movetypes are:
@@ -2809,7 +2870,7 @@ else
   {
   frameheight=height,framewidth=width/num_images;
   if((dupimage=malloc(height*(width/num_images)))==NULL)
-    return;
+    return(0);
   }
 
 vga_clear();
@@ -2863,6 +2924,15 @@ while(!done)
     {
     key=wait_for_keys_or_mouse(zgv_ttyfd);
     
+    /* convert xzgv-ish keypresses */
+    if(cfg.xzgvkeys)
+      switch(key)
+        {
+        case 'b':	key=127; break;
+        case ' ':	key=RK_ENTER; break;
+        case RK_CTRLSPACE: key=' '; break;
+        }
+
     if(has_mouse)
       {
       if(is_end_click_left())
@@ -2871,7 +2941,7 @@ while(!done)
         key=RK_ESC;
       }
 
-    if(key==RK_ENTER)
+    if(key=='n')
       {
       itv.it_value.tv_sec=itv.it_value.tv_usec=0;
       itv.it_interval.tv_sec=itv.it_interval.tv_usec=0;
@@ -2883,9 +2953,16 @@ while(!done)
       {
       case 'p':		paused=!paused; break;
       case RK_ESC:	done=1; break;			/* exit */
+      case RK_ENTER:	done=1; ret=1; break;		/* next image */
+      case ' ':		done=1; ret=2; break;		/* tag & next */
+      case 127: case 8:	done=1; ret=-1; break;		/* prev image */
       }
     }
 
+  hundredths+=delay;
+  if(tagtimeout && hundredths>=tagtimeout)
+    done=1,ret=1;
+  
   /* next frame */
   frame++;
   offset+=(movetype&1)?-1:1;
@@ -2906,4 +2983,6 @@ if(dupimage)
   free(dupimage);
 
 repeat_sig=0;
+
+return(ret);
 }
