@@ -1,5 +1,5 @@
-/* Zgv v2.7 - GIF, JPEG and PBM/PGM/PPM viewer, for VGA PCs running Linux.
- * Copyright (C) 1993-1995 Russell Marks. See README for license details.
+/* zgv 5.2 - GIF, JPEG and PBM/PGM/PPM viewer, for VGA PCs running Linux.
+ * Copyright (C) 1993-2000 Russell Marks. See README for license details.
  *
  * vgadisp.h - prototypes for vgadisp.c
  */
@@ -8,7 +8,7 @@
 /* required for config reading/writing by zgv.c */
 /* pixelsize is also reqd. by readjpeg.c */
 extern int curvgamode,zoom,virtual,vkludge,brightness,pixelsize;
-extern double contrast;
+extern double contrast,picgamma;
 
 /* reqd. by zgv.c:makexv332() */
 extern unsigned char *theimage,*image_palette;
@@ -21,5 +21,6 @@ extern int pic_incr;	/* for ^p and ^n */
  */
 extern int orient_override;
 
-extern int readpicture(char *,hffunc,int,int);
+extern int readpicture(char *giffn,hffunc howfarfunc,int show_dont_tell,
+	int quick,int *real_width,int *real_height);
 extern void aborted_file_cleanup(void);
